/** 
 * Definisce le informazioni memorizzate per un abbonamento:
 * categoria (ordinario, studenti, anziani), durata (in mesi),
 * mese e anno di rilascio.
 */
public class Abbonamento
{
  // Costanti per categoria abbonamento (fascia tariffaria)
  
  /** Costante per categoria abbonamento ordinario. */
  public static final int ORDINARIO = 0x1; // binario 001
  /** Costante per categoria abbonamento per studenti. */
  public static final int STUDENTI = 0x2; // binario 010
  /** Costante per categoria abbonamento per anziani.  */
  public static final int ANZIANI = 0x4; // binario 100

  // Attributi
  
  /** Durata dell'abbonamento, in mesi. */
  public int durata; 
  /** Categoria dell'abbonamento. */
  public int categoria; 
  /** Mese di rilascio dell'abbonamento. */
  int meseRilascio;
  /** Anno di rilascio dell'abbonamento. */
  int annoRilascio;

  // Funzioni ausiliarie per conversione tra codice numerico della
  // categoria e la sua rappresentazione in lettere
    
  /**
   * Ritorna il nome in italiano corrispondente al codice numerico
   * della categoria di abbonamento. Ritorna null se codice non valido.
   * @param  c   codice della categoria di abbonamento, deve essere uno
   *             tra ORDINARIO, STUDENTI, ANZIANI
   */
  public static String categoriaString(int c)
  {
    switch(c)
    {
      case ORDINARIO: return "ordinario";
      case STUDENTI: return "studenti";
      case ANZIANI: return "anziani";
    }
    return null; // non deve accadere
  }

  /**
   * Ritorna codice numerico corrispondente al nome in italiano
   * della categoria di abbonamento. Ritorna 0 se la stringa
   * non rappresenta una categoria.
   */
  public static int categoriaInt(String s)
  {
    if (s.compareTo(categoriaString(ORDINARIO))==0) 
       return ORDINARIO;
    if (s.compareTo(categoriaString(STUDENTI))==0)
       return STUDENTI;
    if (s.compareTo(categoriaString(ANZIANI))==0)
       return ANZIANI;
    return 0; // non deve accadere
  }
  
  // Costruttori

  /**
   * Costruisce un abbonamento con categoria, durata, mese ed
   * anno di rilascio specificati.
   * @param c    categoria, deve essere uno fra ORDINARIO,
   *             STUDENTI, ANZIANI
   * @param d    durata, in mesi
   * @param m    mese di rilascio, da 1 a 12
   * @param a    anno di rilascio
   */
  public Abbonamento(int c, int d, int m, int a)
  {
    setCategoria(c);
    setDurata(d);
    setMeseRilascio(m);
    setAnnoRilascio(a); 
  }

  /**
   * Costruisce un abbonamento con tutti gli attrinuti a valori
   * non specificati ("" le stringhe e 0 i numeri).
   */
  protected Abbonamento()
  {  categoria = durata = meseRilascio = annoRilascio = 0;  }

  /** Costruttore di copia. */
  public Abbonamento(Abbonamento abb)
  {
    categoria = abb.categoria;
    durata = abb.durata;
    meseRilascio = abb.meseRilascio;
    annoRilascio = abb.annoRilascio;
  }
   
  // Funzioni per ritornare gli attributi

  /**
   * Ritorna la categoria dell'abbonamento.
   * @return    categoria da assegnare, deve essere uno fra
   *            ORDINARIO, STUDENTI, ANZIANI
   */
  public int getCategoria() {  return categoria;  }
  /** Ritorna la durata dell'abbonamento, in mesi. */
  public int getDurata() {  return durata;  }
  /** Ritorna il mese di rilascio dell'abbonamento. */
  public int getMeseRilascio() {  return meseRilascio;  }
  /** Ritorna l'anno di rilascio dell'abbonamento. */
  public int getAnnoRilascio() {  return annoRilascio;  }

  // Funzioni per assegnare gli attributi

  /**
   * Assegna la categoria dell'abbonamento.
   * @param c    categoria da assegnare, deve essere uno fra
   *             ORDINARIO, STUDENTI, ANZIANI
   */
  public void setCategoria(int c)  {  categoria = c;  }
  /** Assegna la durata dell'abbonamento, in mesi. */
  public void setDurata(int d)  {  durata = d;  }
  /** Assegna il mese di rilascio dell'abbonamento. */
  public void setMeseRilascio(int m)  {  meseRilascio = m;  }
  /** Assegna l'anno di rilascio dell'abbonamento. */
  public void setAnnoRilascio(int a) {  annoRilascio = a; }
  
  // Funzioni per ritornare informazioni derivate dagli attributi
  
  /** 
   * Ritorna il mese di scadenza dell'abbonamento.
   * Calcolato in base al mese di rilascio e alla durata:
   * ((meseRilascio + durata)%12) + 1.
   */ 
  public int getMeseScadenza()
  {  return  ((meseRilascio + durata - 1)%12) + 1;  }

  /** 
   * Ritorna l'anno di scadenza dell'abbonamento.
   * alcolato in base all'anno di rilascio e alla durata:
   * annoRilascio + (durata/12).
   */   
  public int getAnnoScadenza()
  {  return annoRilascio + ((meseRilascio + durata)/12);  }
   
}
