import java.io.*;

/**
 * Fornisce funzioni (di classe) per leggere un abbonamento da file e
 * ritornarlo, e per scrivere un dato abbonamento su file.<BR>
 * Sono dati il nome del file oppure il file (InputStream o PrintStream)
 * gia' aperto.<BR>
 * Stabilisce anche la sintassi dei file che rappresentano
 * gli abbonamenti, che e' la seguente:<BR>
 * <TT>ABBONATO<BR>
 * Cognome: _stringa_<BR>
 * Nome: _stringa_<BR>
 * Anno: _intero_<BR>
 * Indirizzo: _sequenza_di_stringhe_<BR>
 * Telefono: _stringa_<BR>
 * Email: _stringa_<BR>
 * Categoria: _stringa_<BR>
 * Durata: _intero_<BR>
 * Rilascio: _due_interi_<BR>
 * Password: _stringa_<BR></TT>
 * dove:<BR>
 * - la stringa che esprime la categoria deve essere una fra: "ordinario",
 *   "studenti", "anziani";<BR>
 * - i due interi che esprimono la data di rilascio corrispondono
 *   nell'ordine a mese (da 1 a 12) e anno;<BR>
 * - memorizzare la password in chiaro sul file e' stupido!
 *   ma facciamo cosi' per non complicarci la vita.
 */
public class LeggiScriviAbbonati
{
  // Costanti per le parole chiave presenti nel file
  
  /** Parola chiave in testa al file. */
  public static final String P_ABBONATO = "ABBONATO";
  /** Parola chiave per cognome. */
  public static final String P_COGNOME = "Cognome:";
  /** Parola chiave per nome. */
  public static final String P_NOME = "Nome:";
  /** Parola chiave per anno. */
  public static final String P_ANNO = "Anno:";
  /** Parola chiave per indirizzo. */
  public static final String P_INDIRIZZO = "Indirizzo:";
  /** Parola chiave per telefono. */
  public static final String P_TELEFONO = "Telefono:";
  /** Parola chiave per email. */
  public static final String P_EMAIL = "Email:";
  /** Parola chiave per categoria. */
  public static final String P_CATEGORIA = "Categoria:";
  /** Parola chiave per durata. */
  public static final String P_DURATA = "Durata:";
  /** Parola chiave per rilascio. */
  public static final String P_RILASCIO = "Rilascio:";
  /** Parola chiave per password. */
  public static final String P_PASSWORD = "Password:";

  // Funzioni ausiliarie di lettura
  
  /* Legge la prossima stringa e controlla che sia uguale alla
     keyword, se non lo e' fa eccezione */
  private static void leggiKeyword(MyReader reader, String key) 
          throws Exception
  {
    String aux = reader.readString();
    if (!reader.isKeyword(aux,key))
      throw new Exception("Cannot find " + key + " keyword");
  }

  /* Legge tutte le stringhe fino ad incontrare la keyword,
     se non la trova fa eccezione, memorizza la concatenazione
     (con spazi inframezzati) di tutte le stringhe lette,
     keyword esclusa, e la ritorna come risultato */
  private static String leggiFinoKeyword(MyReader reader, String key) 
          throws Exception
  {
    StringBuffer sb = new StringBuffer();
    String st = reader.readString();
    while (st.compareTo(key)!=0)
    {
      if (sb.length()==0) sb.append(st);
      else sb.append(" " + st);
      st = reader.readString();
      if (st==null) // ho raggiunto fine file
        throw new Exception("Cannot find " + key + " keyword");
//System.out.println("leggiFinoKeyword " + key + " Finora letto: ["+ (new String(sb)) + "]");
   }
    return new String(sb);
  }
  
  /* Legge abbonamento usando classe MyReader. */
  private static Abbonato leggiAux(MyReader reader) throws Exception
  {
    String co, no, in, te, em, ca, pw;
    int an, du, mr, ar;
    // leggo tutto
    leggiKeyword(reader,P_ABBONATO);
    leggiKeyword(reader,P_COGNOME);
    co = leggiFinoKeyword(reader,P_NOME);
    no = leggiFinoKeyword(reader,P_ANNO);
    an = reader.readInteger();
    leggiKeyword(reader,P_INDIRIZZO);
    in = leggiFinoKeyword(reader,P_TELEFONO);
    te = leggiFinoKeyword(reader,P_EMAIL);
    em = leggiFinoKeyword(reader,P_CATEGORIA);
    ca = leggiFinoKeyword(reader,P_DURATA);
    du = reader.readInteger();
    leggiKeyword(reader,P_RILASCIO);
    mr = reader.readInteger();
    ar = reader.readInteger();
    leggiKeyword(reader,P_PASSWORD);
    pw = reader.readString();
    // costruisco oggetto
    Persona p = new Persona(co,no,an);
    p.setIndirizzo(in);
    p.setTelefono(te);
    p.setEmail(em);
    Abbonamento a = new Abbonamento(Abbonamento.categoriaInt(ca),du,mr,ar);
    Abbonato res = new Abbonato(p,a);
    res.setPassword(pw);
    return res;
  }
  
  // Funzioni di lettura pubbliche
  
  /** 
   * Dato un nome di file, legge da esso un abbonato e lo ritorna.
   * @param  nomeFile    nome del file di input.
   */
  public static Abbonato leggi(String nomeFile) throws Exception
  {  return leggiAux(new MyReader(nomeFile));  }  

  /** 
   * Dato un file gia' aperto, legge da esso un abbonato e lo ritorna.
   * @param  f  file di input (classe InputStream).
   */
  public static Abbonato leggi(InputStream f) throws Exception
  {  return leggiAux(new MyReader(f));  }  
  
  // Funzioni di scrittura pubbliche
  
  /**
   * Funzione che scrive un abbonato su file di cui viene dato il nome.
   * @param  abb       l'abbonato da scrivere  
   * @param  nomeFile  nome del file su cui scriverlo
   */
  public static void scrivi(Abbonato abb, String nomeFile)
         throws Exception
  { 
    FileOutputStream f = new FileOutputStream(nomeFile);
    scrivi(abb,new PrintStream(f));
  }

  /**
   * Funzione che scrive un abbonato un file gia' aperto.
   * @param  abb      l'abbonato da scrivere
   * @param  printer  il file su cui scriverlo
   */
  public static void scrivi(Abbonato abb, PrintStream printer)
         throws Exception
  { 
    printer.println(P_ABBONATO);
    printer.println(P_COGNOME + " " + abb.titolare.getCognome());
    printer.println(P_NOME + " " + abb.titolare.getNome());
    printer.println(P_ANNO + " " + abb.titolare.getAnnoNascita());
    printer.println(P_INDIRIZZO + " " + abb.titolare.getIndirizzo());
    printer.println(P_TELEFONO + " " + abb.titolare.getTelefono());
    printer.println(P_EMAIL + " " + abb.titolare.getEmail());
    printer.println(P_CATEGORIA + " " +
         Abbonamento.categoriaString(abb.titolo.getCategoria()));
    printer.println(P_DURATA + " " + abb.titolo.getDurata());
    printer.println(P_RILASCIO + " " + abb.titolo.getMeseRilascio() +
         " " + abb.titolo.getAnnoRilascio());
    printer.println(P_PASSWORD + " " + abb.password);
  }

}
