/***********************************************************************/

import java.io.*;
import java.util.*;

/**
 * Classe che ingloba un file e fornisce alcune funzioni per leggere
 * stringhe, parole chiave, interi e double da tale file.
 * La classe Java FileInputStream (che e' anche un InputStream) 
 * non fornisce funzioni a questo scopo (devo leggere byte per byte).
 */

public class MyReader 
{
  private InputStream fd; // il file di input da cui leggere

/** Apre file con nome dato e crea reader su quel file. */
  public MyReader(String name) throws IOException
  {  fd = new FileInputStream(name);  }

/** Crea reader su file gia' aperto, per esempio System.in. */
  public MyReader(InputStream ffdd) {  fd = ffdd;  }

/** Salta spazi e ritorna codice del primo carattere non 
 * spazio (-1 se fine file). */
  protected int skipWhitespace() throws IOException
  {
    int cod = (int)(' ');
    while (Character.isWhitespace( (char)cod ))
    {
      cod = fd.read();
      if (cod == -1) return -1;
    } 
    return cod;
  }

/** Legge una stringa e la ritorna, lascia il file posizionato sul
 *  primo carattere seguente. */
  public String readString() throws IOException
  {
    StringBuffer s = new StringBuffer("");
    int cod = skipWhitespace();
    while ( (cod!=-1) && !Character.isWhitespace( (char)cod ))
    {
      s.append( (char)cod );
      cod = fd.read();
    }
    return new String(s);
  }

/** Controlla uguaglianza tra una stringa e una parola
 * chiave assegnata, ritorna true se uguale e false se diversa. */
  public boolean isKeyword(String s, String kw)
  {
    return (kw.compareTo(s) == 0);
  }

/** Legge un intero e lo ritorna, lascia il file posizionato sul
 * primo carattere seguente. */
  public int readInteger() throws IOException, NumberFormatException
  {
    String s = readString();
    if (s.length()==0) throw new IOException("Integer expected");
    return Integer.valueOf(new String(s)).intValue();
  } 

/** Legge un numero reale e lo ritorna, lascia il file posizionato sul
 * primo carattere seguente. */
  public double readDouble() throws IOException, NumberFormatException
  {
    String s = readString();
    if (s.length()==0) throw new IOException("Float expected");
    return Double.valueOf(new String(s)).doubleValue();
  } 

/** Legge fino a fine file e ritorna tutto come una sola stringa. */
  public String readAll() throws IOException
  {
    StringBuffer s = new StringBuffer("");
    int cod = fd.read();
    while (cod!=-1)
    {
      s.append( (char)cod );
      cod = fd.read();
    }
    return new String(s);
  }

  /** Legge fino a che non trova la parola chiave, restituisce quello che
   * ha letto un un'unica stringa. */
  public String readUntilKeyword(String key) throws IOException
  {
    String s = new String();
    StringBuffer aux = new StringBuffer();
    while (!isKeyword(s,key))
    {  aux.append(" "); aux.append(s); s = readString();  }
    if (aux.length()==0) return null;
    return new String(aux);
  }

};

/***********************************************************************/

