/**
 * Realizza un utente generico del data base.
 * Memorizza il data base, un abbonato correntemente selezionato
 * (caricato dal data base e/o da scrivere nello stesso), un booleano
 * che dice se l'abbonato corrente e' stato modificato e quindi 
 * necessita di essere salvato.
 * Super-classe di UtenteAbbonato e di UtenteGestore.
 */
public class Utente
{
  // Attributi
  
  /** Il data base usato. */
  protected DataBase dataBase;
  /** L'abbonato correntemente selezionato. */  
  protected Abbonato corrente;
  /** booleano che value true sse l'abbonato correntemente selezionato
   * e' stato modificato e quindi deve essere salvato. */
  protected boolean modificato;

  // Costruttore
  
  /** 
   * Costruisce Utente data la directory del data base, usata 
   * internamente per costruire il data base. Inizialmente
   * nessun abbonato e' selezionato e non ci sono state modifiche.
   * @param  nomeDir   nome della directory contenente i file del
   *                   data base
   */
  public Utente(String nomeDir) throws Exception
  {
    dataBase = new DataBase(nomeDir);
    corrente = null;
    modificato = false;
  }

  // Funzioni relative all'abbonato selezionato e suo stato
  
  /** Restituisce true sse esiste un abbonato correntemente selezionato. */
  public boolean haSelezionato() { return (corrente!=null);  }
  /** Restituisce true sse l'abbonato correntemente selezionato
   * e' stato modificato. */
  public boolean haModificato() {  return (modificato);  }
  /** Restituisce una copia dell'abbonato selezionato. */
  public Abbonato copiaSelezionato()
  {
    if (corrente==null) return null;
    else return new Abbonato(corrente);
  }
  /** Deseleziona l'eventuale abbonato selezionato. */
  public void deseleziona()
  {  corrente = null; modificato = false;  }

  /**
   * Aggiorna l'abbonato selezionato coi dati memorizzati nella
   * coppia (persona,abbonamento) data. Aggiorna solo i campi che 
   * risultano modificabili (qui indirizzo, telefono, email).
   */
  public void aggiorna(InfoBase info) throws Exception
  {  
    if (indirizzo().compareTo(info.titolare.getIndirizzo())!=0)
          cambiaIndirizzo(info.titolare.getIndirizzo());
    if (telefono().compareTo(info.titolare.getTelefono())!=0)
          cambiaTelefono(info.titolare.getTelefono());
    if (email().compareTo(info.titolare.getEmail())!=0)
          cambiaEmail(info.titolare.getEmail());
  }

  /** Salva nel data base l'abbonato selezionato. */
  public void salva() throws Exception 
  {
    if ( (corrente!=null) && modificato ) dataBase.salva(corrente);
    modificato = false;
  }

  /** Ricarica dal data base l'abbonato selezionato. */
  public void ricarica() throws Exception
  {
    if (corrente!=null)
    {
      Criterio cri = new Criterio();
      cri.titolare.setCognome(corrente.titolare.getCognome());
      cri.titolare.setNome(corrente.titolare.getNome());
      cri.titolare.setAnnoNascita(corrente.titolare.getAnnoNascita());
      Abbonato[] aux = dataBase.carica(cri);
      if (aux.length!=1) 
         throw new Exception("Abbonato da ricaricare non presente");
      corrente = aux[0];
      modificato = false;
    }
  }
  
  // Funzioni che restituiscono le informazioni dell'abbonato selezionato
  
  /** Ritorna il cognome dell'abbonato correntemente selezionato. */
  public String cognome() {  return corrente.titolare.getCognome();  }
  /** Ritorna il nome dell'abbonato correntemente selezionato. */
  public String nome() {  return corrente.titolare.getNome();  }
  /** Ritorna l'anno di nascita dell'abbonato correntemente selezionato. */
  public int annoNascita() {  return corrente.titolare.getAnnoNascita();  }
  /** Ritorna l'eta' dell'abbonato correntemente selezionato, come
   * differenza tra anno corrente e anno di nascita
   * @param   a l'anno corrente */
  public int eta(int a) {  return corrente.titolare.getEta(a);  }
  
  /** Ritorna l'indirizzo dell'abbonato correntemente selezionato. */
  public String indirizzo() {  return corrente.titolare.getIndirizzo();  }
  /** Ritorna il telefono dell'abbonato correntemente selezionato. */
  public String telefono() {  return corrente.titolare.getTelefono();  }
  /** Ritorna l'email dell'abbonato correntemente selezionato. */
  public String email() {  return corrente.titolare.getEmail();  }

  /** Ritorna la categoria di abbonamento dell'abbonato selezionato. */
  public int categoria() {  return corrente.titolo.getCategoria();  }
  /** Ritorna la durata dell'abbonamento dell'abbonato selezionato. */
  public int durata() {  return corrente.titolo.getDurata();  }
  /** Ritorna il mese di rilascio dell'abbonamento dell'abbonato selezionato. */
  public int meseRilascio() {  return corrente.titolo.getMeseRilascio();  }
  /** Ritorna l'anno di rilascio dell'abbonamento dell'abbonato selezionato. */
  public int annoRilascio() {  return corrente.titolo.getAnnoRilascio();  }

  /** Ritorna il mese di scadenza dell'abbonamento dell'abbonato selezionato. */
  public int meseScadenza() {  return corrente.titolo.getMeseScadenza();  }
  /** Ritorna l'anno di scadenza dell'abbonamento dell'abbonato selezionato. */
  public int annoScadenza() {  return corrente.titolo.getAnnoScadenza();  }
  
  // Funzioni per modificare l'abbonato selezionato
    
  /** Assegna l'indirizzo dell'abbonato correntemente selezionato. */
  public void cambiaIndirizzo(String s)
  {  corrente.titolare.setIndirizzo(s); modificato = true;  }
  /** Assegna il telefono dell'abbonato correntemente selezionato. */
  public void cambiaTelefono(String s)
  {  corrente.titolare.setTelefono(s); modificato = true;  }
  /** Assegna l'email dell'abbonato correntemente selezionato. */
  public void cambiaEmail(String s)
  {  corrente.titolare.setEmail(s); modificato = true;   }
  /** Assegna la password dell'abbonato correntemente selezionato. */
  public void cambiaPassword(String s)
  {  corrente.setPassword(s); modificato = true;   }
}
