/**
 * Sotto-classe di Utente, memorizza in piu':
 * il criterio di ricerca corrente;
 * un array di oggetti abbonato, che viene riempito ogni volta coi
 * risultati dell'ultima ricerca.<BR>
 * L'attributo abbonato corrente, ereditato dalla super-classe,
 * e' usato per memorizzare l'abbonato selezionato (scelto tra quelli
 * caricati nell'array, o uno nuovo).<BR>
 * Ha funzioni per:
 * caricare dal data base gli abbonati che soddisfano un certo
 * criterio di ricerca, 
 * ritornare il numero di risultati ottenuti,
 * selezionare l'i-esimo risultato, oppure   
 * creare un nuovo abbonato (specificando i suoi dati, il nuovo abbonato
 * viene subito selezionato, modificare nell'abbonato selezionato anche
 * categoria e durata dell'abbonamento, mese e anno di rilascio...
 */
public class UtenteGestore extends Utente
{

  /** Criterio di ricerca corrente. */
  protected Criterio criterioRicerca;

  /** Array che memorizza gli abbonati risultati della ricerca. */  
  protected Abbonato[] caricati;

  // Costruttore
  
  /**
   * Costruisce utente gestore data la directory del data base, usata 
   * internamente per costruire il data base. Inizialmente
   * nessun abbonato e' selezionato e non ci sono state modifiche,
   * nessun criterio di ricerca e' impostato, nessuna ricerca e'
   * stata fatta e non ci sono risultati caricati.
   * @param  nomeDir   nome della directory contenente i file del
   *                   data base
   */             
  public UtenteGestore(String nomeDir) throws Exception
  {
    super(nomeDir);
    criterioRicerca = null;
    caricati = null;
  }

  // Funzioni relative a ricerca
    
  /** Assegna il criterio di ricerca corrente. */
  public void setCriterio(Criterio crit)
  {  criterioRicerca = crit;  }
  
  /**
   * Cerca nel data base secondo il criterio assegnato, memorizza
   * gli abbonati risultati dalla ricerca nell'array. Se non e' stato
   * imposto un criterio di ricerca, none carica nulla.
   * Se invece il criterio di ricerca ha campi vuoti, carica tutti
   * gli abbonati del data base.
   * @see UtenteGestore#caricati
   */
  public void ricerca() throws Exception
  {
    if (criterioRicerca!=null) 
       caricati = dataBase.carica(criterioRicerca);
    else caricati = null;
  }
  
  /** Ritorna il numero di abbonati risultati della ricerca. */
  public int numeroRisultati()
  { if (caricati==null) return 0; else return caricati.length;  }     

  /** Seleziona il risultato i-esimo come abbonato corrente.
   * Poi vi potranno agire le modifiche. */
  public void selezionaRisultato(int i)
  {
    if ( (i<0) || (i>=numeroRisultati()) ) corrente = null;
    else corrente = caricati[i];
    modificato = false;
  }
  
  // Funzioni per nuovo abbonato
  
  /** Crea nuovo abbonato e lo seleziona come abbonato
   * corrente. Poi vi potranno agire le modifiche. */
  public void creaNuovo(String cognome, String nome, int anno)
         throws Exception
  {
    // Prima assicurati che non ci sia gia' questa persona
    Criterio cr = new Criterio();
    cr.titolare.setCognome(cognome);
    cr.titolare.setNome(nome);
    cr.titolare.setAnnoNascita(anno);
    Abbonato[] ris = dataBase.carica(cr);
    if (ris.length!=0) 
       throw new Exception("La persona ha gia' un abbonamento");
    corrente = new Abbonato(new Persona(cognome,nome,anno),
                            new Abbonamento());
    modificato = true;
  }

  /**
   * Aggiorna l'abbonato selezionato coi dati memorizzati nella
   * coppia (persona,abbonamento) data. Re-implementata dalla
   * super-classe Utente, aggiorna anche altri campi.
   * Inoltre crea al volo l'abbonato selezionato, se non esiste.
   */
  public void aggiorna(InfoBase info) throws Exception
  { 
    if (!haSelezionato())
       creaNuovo(info.titolare.getCognome(), info.titolare.getNome(),
                 info.titolare.getAnnoNascita()); 
    super.aggiorna(info);
    if (categoria()!=info.titolo.getCategoria())
          cambiaCategoria(info.titolo.getCategoria());
    if (durata()!=info.titolo.getDurata())
          cambiaDurata(info.titolo.getDurata());
    if (meseRilascio()!=info.titolo.getMeseRilascio())
          cambiaMeseRilascio(info.titolo.getMeseRilascio());
    if (annoRilascio()!=info.titolo.getAnnoRilascio())
          cambiaAnnoRilascio(info.titolo.getAnnoRilascio());
  }
  
  // Funzioni che modificano l'abbonato correntemente selezionato
  
  /** Assegna la categoria dell'abbonamento selezionato. */
  public void cambiaCategoria(int c)
  {  corrente.titolo.setCategoria(c); modificato = true;   }
  /** Assegna la durata dell'abbonamento selezionato. */  
  public void cambiaDurata(int d)
  {  corrente.titolo.setDurata(d); modificato = true;   }
  /** Assegna il mese di rilascio dell'abbonamento selezionato. */
  public void cambiaMeseRilascio(int m)
  {  corrente.titolo.setMeseRilascio(m); modificato = true;   }
  /** Assegna l'anno di rilascio dell'abbonamento selezionato. */
  public void cambiaAnnoRilascio(int a)
  {  corrente.titolo.setAnnoRilascio(a); modificato = true;   }

}
